#line 71 "dmcs.Rnw"
Stangle('dmcs.Rnw', annotate=FALSE) # make dmcs.R for later inclusion

# collimator position vs. time, linear
Jc <- function(t, Jci=0.04, Jcf=0.05, dt=1){
  Jc <- 0*t + Jci # initial value
  subset <- dt<=t
  Jc[subset] <- Jcf # final value
  subset <- 0<t & t<dt
  Jc[subset] <- Jci + (Jcf-Jci)*t[subset]/dt # collimator moving
  return(Jc)
}

# slope of distribution vs. time, linear
Ac <- function(t, Ai=1, Af=0.9, dt=1){
  Ac <- 0*t + Ai # initial value
  subset <- dt<=t
  Ac[subset] <- Af # final value
  subset <- 0<t & t<dt
  Ac[subset] <- Ai + (Af-Ai)*t[subset]/dt # collimator moving
  return(Ac)
}

# initial, final, and asymptotic distributions, linear
flin <- function(J, A=1, J0=0.04){
  f <- 0*J # default outside range
  J0 <- abs(J0) # define edge as positive
  subset <- abs(J)<J0 # triangular with cusp at J=0
  f[subset] <- A*(J0 - abs(J[subset])) # linear
  return(f) }

# evolution of the distribution function (der=0)
# or its derivative (der=1), analytical formulas
fs <- function(J, t, Ai=1, Af=0.9, Jci=0.04, Jcf=0.05, D=1e-5, dt=1, der=0) {
  if(der!=0 & der!=1) stop("Derivative ('der') must be 0 or 1.")
  fs <- 0 # default outside collimators
  if(Jci>Jcf) IN<-TRUE else IN<-FALSE # inward or outward step?
  if(t<0){ # initial distribution
    if(der==0) fs <- flin(J, A=Ai, J0=Jci)
    if(der==1) fs <- -Ai
  } else{ # evolution
    Jco <- Jc(t, Jci=Jci, Jcf=Jcf, dt=dt) # collimator position
    Aco <- Ac(t, Ai=Ai, Af=Af, dt=dt) # slope of asymptotic solution
    if(0<=J & J<=Jco){ # inside collimators
      s <- sqrt(2*D*t) # Gaussian sigma
      # terms common to both inward and outward step
      if(der==0) fs.base <- -pnorm(-J/s) * (Ai*(Jci-J) - Aco*(Jco-J)) +
          pnorm((J-2*Jco)/s) * (Ai*(Jci+J-2*Jco) - Aco*(J-Jco)) +
          s/sqrt(2*pi) * (
            exp(-0.5*(J/s)^2) * (Aco-Ai) +
            (-1)*exp(-0.5*((2*Jco-J)/s)^2) * (Aco-Ai) )
      if(der==1) fs.base <- (Ai-Aco)*(pnorm(-J/s) + pnorm((J-2*Jco)/s)) +
          (Ai*Jci-Aco*Jco)/sqrt(2*pi)/s *
              (exp(-0.5*(J/s)^2)+exp(-0.5*((J-2*Jco)/s)^2))
      # add specific terms
      if(IN) { # inward step
        if(der==0) fs <- fs.base +
            (-1)*Ai*(Jci+J-2*Jco) +
            2 * pnorm((Jco-J)/s) * Ai * (Jci-Jco)
        if(der==1) fs <- fs.base +
            (-1)*Ai + 2*Ai*(Jco-Jci)*exp(-0.5*((Jco-J)/s)^2)/sqrt(2*pi)/s
      } else { # outward step
        if(der==0) fs <- fs.base +
          pnorm((Jci-J)/s) * Ai * (Jci-J) +
          (-1)*pnorm((Jci+J-2*Jco)/s) * Ai * (Jci+J-2*Jco) +
          s/sqrt(2*pi) * (
            exp(-0.5*((Jci-J)/s)^2) * Ai +
            (-1)*exp(-0.5*((Jci+J-2*Jco)/s)^2) * Ai )
        if(der==1) fs <- fs.base +
            (-1)*Ai*(pnorm((Jci-J)/s)+pnorm((Jci+J-2*Jco)/s))
      }
    }
  }
  return(fs)
}
fv <- Vectorize(fs, vectorize.args=c("J","t")) # vectorize the function

# loss rate vs. time
# particle flux at the collimator L = -D * (df/dJ)|J=Jc
# with conversion constant k and background B
loss.rate <- function(t, Ai=1, Af=0.9, Jci=0.04, Jcf=0.05, D=1e-5, dt=1,
                      k=1, B=0) {
  L <- -Ai + 0*t # default before step
  subset <- t>=0 # evolution
  s <- sqrt(2*D*t[subset]) # Gaussian sigmas
  Jco <- Jc(t[subset], Jci=Jci, Jcf=Jcf, dt=dt) # collimator positions
  Aco <- Ac(t[subset], Ai=Ai, Af=Af, dt=dt) # slopes
  if(Jci>Jcf) IN<-TRUE else IN<-FALSE # inward or outward?
  # common terms
  L[subset] <- 2*(Ai-Aco)*pnorm(-Jco/s) +
    2/sqrt(2*pi)/s * (Ai*Jci-Aco*Jco) * exp(-0.5*(Jco/s)^2)
  if(IN) {
      L[subset] <- L[subset] - Ai - 2/sqrt(2*pi)/s * Ai * (Jci-Jco)
  } else {
      L[subset] <- L[subset] - 2 * Ai * pnorm((Jci-Jco)/s) }
  return(-k*D*L+B)
}

# sample data
# read collimator position and loss rate data
ReadACNETData <- function(fn="CFBI_NG.txt.gz") {
cat("Reading file ", fn, "...\n")
dn <- gsub(".txt.gz", "", fn) # name of data frame
t0 <- scan(fn, what="character", skip=1, nlines=1) # get initial time
t0 <- t0[6]
h <- as.numeric(substr(t0,1,2))
m <- as.numeric(substr(t0,4,5))
s <- as.numeric(substr(t0,7,8))
assign("T0", h+m/60+s/3600, envir=.GlobalEnv)
cat("Start time: ", T0, " h\n")
cnam <- scan(fn, what="character", skip=2, nlines=1, sep="\t") # column names
cnam <- substr(cnam, 3, 12) # strip logger names
cnam <- gsub("[^[:alnum:]]", "", cnam) # remove non alphanumeric characters
if(length(cnam)<72) cnam <- gsub("0", "", cnam) # remove 0s from scalars
cnam[seq(1,length(cnam),by=2)] <-
  paste("t", cnam[seq(2,length(cnam),by=2)], sep="") # times
assign(dn, read.table(fn, header=FALSE, skip=3, colClasses="numeric",
                      na.strings=c("NA","999999"), col.names=cnam),
       envir= .GlobalEnv) # fill data frame
}
ReadACNETData('Store8749_CP.txt.gz')
ReadACNETData('Store8749_TLF480_15Hz.txt.gz')

tc <- Store8749_CP$tF48VCP/3600 + T0 # time stamps of coll. positions [h]
cpmil <- Store8749_CP$F48VCP # collimator position [mils]
cp0 <- 366 # collimator offset (arbitrary)
cp <- (cpmil - cp0) * 25.4 # collimator position [um]

tT1 <- Store8749_TLF480_15Hz$tLF48/3600 + T0 # time stamps of losses [h]
T1 <- Store8749_TLF480_15Hz$LF48 # loss monitor [Hz]

# plot setup
options(SweaveHooks=list(fig=function() { # common plot options
    pt<-10; fo<-"Times"
    ps.options(pointsize=pt, family=fo)
    pdf.options(pointsize=pt, family=fo)
    par(oma=rep(0,4), mar=c(4.2,4.2,0.2,0.2)) }))


#line 217 "dmcs.Rnw"
ti <- 9.105 # initial time [h]
tf <- 9.294 # final time
plot.unit <- 60
xl <- c(0, tf-ti)*plot.unit
par(mar=c(0, 3.2, 0, 0), oma=c(3.2, 0, 0.2, 0.2), mgp=c(2,1,0), tcl=-0.25)
layout(matrix(1:2, nrow=2, ncol=1))
x <- subset((tT1-ti)*plot.unit, ti<=tT1 & tT1<=tf) # time stamps and loss rates
y <- subset( T1, ti<=tT1 & tT1<=tf)
ii <- round(seq(1, length(x), length=2048)) # choose a few representative points
x <- x[ii]; y <- y[ii]
plot(x, y, type="l", xlim=xl, axes=FALSE, xlab="",
     ylab="Loss rate [arb. units]")
axis(2); box()
x <- subset((tc-ti)*plot.unit, ti<=tc & tc<=tf) # collimator positions
y <- subset(cp, ti<=tc & tc<=tf)
plot(x, y, type="o", pch=20, xlim=xl, xlab="",
     ylab=expression(paste("Collimator position [", mu*m, "]")))
mtext("Time [min]", side=1, line=2, outer=TRUE)


#line 365 "dmcs.Rnw"
# sample parameters
Jci <- 0.040; Jcf <- 0.050 # initial and final collimator actions [um]
dt <- 1 # time to move collimator from Jci to Jcf [s]
Ai <- 1 # initial slope of distribution function [um^-2]
Af <- Ai*(Jci/Jcf) # final slope
tt <- seq(0, dt, length=5) # choose times
JJ <- Jc(tt, Jci=Jci, Jcf=Jcf, dt=dt) # calculate coll. pos.
AA <- Ac(tt, Ai=Ai, Af=Af, dt=dt) # calculate slopes
co <- gray(seq(0.6, 0, length=length(tt))) # define colors, grayscale
Jmin <- 0; Jmax <- max(Jci,Jcf) # plot range
plot(c(Jmin,Jmax),
     c(0, max(Jci*Ai,Jcf*Af)),
     type="n",
     xlab=expression(paste("Action [", mu, "m]", sep="")),
     ylab=expression(paste("Distribution function [", mu*m^{-1}, "]", sep="")))
abline(v=JJ, col=co, lwd=2) # plot collimator positions
for(i in seq(along=tt)){ # plot asymtotic functions
  curve(flin(x, A=AA[i], J0=JJ[i]), Jmin, Jmax, add=TRUE, col=co[i], lwd=2) }
legend("bottomleft", bty="n", # describe times for each curve
       title="Time [s]", title.col="black",
       legend=format(tt, di=2), col=co, text.col=co)
xx <- 0.5*(Jmin+Jmax) # labels curves
text(xx, flin(xx, A=Ai, J0=Jci), expression(f[0](J)), pos=2)
text(xx, flin(xx, A=Af, J0=Jcf), expression(f[infinity](J)), pos=4)
xx <- 0.6*Jmax; yy <- 0.1*max(Jci*Ai, Jcf*Af)
text(xx, yy, expression(f[a](J,t)), pos=2)
xx2 <- 0.7*Jmax
lines(x=c(xx, xx2), y=c(yy, flin(xx2, A=AA[2], J0=JJ[2])))
caption.text <- "no" # text for LaTeX caption
if(Jcf<Jci) caption.text <- "inward"
if(Jcf>Jci) caption.text <- "outward"


#line 554 "dmcs.Rnw"
# sample parameters
J1 <- 0.040; J2 <- 0.050 # initial and final collimator actions [um]
dt <- 1 # time to move collimator from Jci to Jcf [s]
A1 <- 1 # initial slope of distribution function [um^-2]
A2 <- A1*(J1/J2) # final slope
D <- 1e-5 # diffusion coefficient [um^2/s]
# choose times to plot
tscale.short <- abs(J1-J2)^2/D
tscale.long <- min(J1,J2)^2/D
tt <- c(seq(0, dt, length=5), # collimator moving
        c(3*dt, tscale.short, 2*tscale.short), # short term after
        c(5*tscale.short, tscale.long, 2*tscale.long)) # long term after
co <- gray(seq(0.6, 0, length=length(tt))) # define colors, grayscale
Jmin <- 0; Jmax <- max(J1,J2) # plot range
layout(matrix(1:2, nrow=1, ncol=2)) # plot layout
for(i in 1:2) {
  if(i==1){ # left plot, inward
    Jci<-J2; Jcf<-J1; Ai<-A2; Af<-A1}
  if(i==2){ # right plot, outward
    Jci<-J1; Jcf<-J2; Ai<-A1; Af<-A2}
  JJ <- Jc(tt, Jci=Jci, Jcf=Jcf, dt=dt) # calculate coll. pos.
  AA <- Ac(tt, Ai=Ai, Af=Af, dt=dt) # calculate slopes
  ylabel <- expression(paste("Distribution function [", mu*m^{-1}, "]", sep=""))
  if(i==2) ylabel <- ""
  plot(c(Jmin,Jmax),
     c(0, max(Jci*Ai,Jcf*Af)),
     type="n",
     xlab=expression(paste("Action [", mu, "m]", sep="")),
     ylab=ylabel)
abline(v=JJ, col=co, lwd=2) # plot collimator positions
for(i in seq(along=tt)){ # plot asymptotic functions
  curve(fv(J=x, t=tt[i], Ai=Ai, Af=Af, Jci=Jci, Jcf=Jcf, D=D, dt=dt),
        Jmin, Jmax, add=TRUE, col=co[i], lwd=2) }
legend("bottomleft", bty="n", # describe times for each curve
       title="Time [s]", title.col="black",
       legend=formatC(tt, drop0=TRUE), col=co, text.col=co, cex=0.8)
}


#line 668 "dmcs.Rnw"
# sample parameters
J1 <- 0.040; J2 <- 0.050 # initial and final collimator actions [um]
dt <- 1 # time to move collimator from Jci to Jcf [s]
D <- c(1e-6, 1e-5, 1e-4) # show the effect of different diffusion coeff.
k <- 1; B <- 0
tscale.short <- abs(J1-J2)^2/min(D) # estimate time scales
tscale.long <- min(J1,J2)^2/min(D)
tmax <- tscale.short/2
co <- gray(seq(0.5, 0, length=length(D))) # define colors, grayscale
Jmin <- 0; Jmax <- max(J1,J2) # plot range
layout(matrix(1:2, nrow=1, ncol=2)) # plot layout
for(i in 1:2) {
  if(i==1){ # left plot, inward
    Jci<-J2; Jcf<-J1
    ylabel <- "Loss rate [arb. units]"
    legend.pos <- "topright"}
  if(i==2){ # right plot, outward
    Jci<-J1; Jcf<-J2
    ylabel <- ""
    legend.pos <- "bottomright"}
  ADD <- FALSE
  for(j in 1:length(D)){
    Ai <- 1/D[j] # scale slope to keep steady-state rates constant
    Af <- Ai*Jci/Jcf
    if(j>1) ADD <- TRUE
    curve(loss.rate(t=x, Jci=Jci, Jcf=Jcf, Ai=Ai, Af=Af, D=D[j],
                    dt=dt, k=k, B=B), -0.1*tmax, tmax,
          xlab="Time [s]", ylab=ylabel, add=ADD, lwd=2, col=co[j])}
  if(i==1) legend(legend.pos, bty="n", # print diffusion rate for each curve
     title=expression(paste("Diffusion rate [", mu*m^2/s, "]")),
     title.col="black",
     legend=formatC(D, format="E", di=1), col=co, text.col=co)
}


